/************************************************************
 *  * Hyphenate CONFIDENTIAL
 * __________________
 * Copyright (C) 2016 Hyphenate Inc. All rights reserved.
 *
 * NOTICE: All information contained herein is, and remains
 * the property of Hyphenate Inc.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Hyphenate Inc.
 */

#import "UIViewController+HUD.h"
#import <objc/runtime.h>
#import "MBProgressHUD+Add.h"

static const float delay_seconds = 2.5;
static const void *HttpRequestHUDKey = &HttpRequestHUDKey;


@implementation UIViewController (HUD)
@dynamic HUD;

- (MBProgressHUD *)HUD{
    return objc_getAssociatedObject(self, HttpRequestHUDKey);
}

- (void)setHUD:(MBProgressHUD *)HUD{
    objc_setAssociatedObject(self, HttpRequestHUDKey, HUD, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (void)showHudInView:(UIView *)view hint:(NSString *)hint{
    
    if ([hint isKindOfClass:[NSNull class]]) {
        NSLog(@"hint is null......");
        hint = @"";
    }
    [MBProgressHUD showMessage:hint toView:view];
    
}

- (void)showHint:(NSString *)hint delay:(CGFloat)delay
{
    UIView *view = [UIApplication sharedApplication].keyWindow;
    
    if ([hint isKindOfClass:[NSNull class]]) {
        NSLog(@"hint is null......");
        hint = @"";
    }
    [MBProgressHUD showMessage:hint view:view];
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(2 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [MBProgressHUD hideHUDForView:view animated:YES];
    });
}
- (void)showHint:(NSString *)hint
{
    if ([hint isKindOfClass:[NSNull class]]) {
        NSLog(@"hint is null......");
        return;
    }
    [self hideHud];
    [self showHint:hint delay:delay_seconds];
}
// yhy added this method
-(void)showDetail:(NSString*)text{
    [self showDetail:text delay:delay_seconds fontSize:15];
}
// yhy added this method
-(void)showDetail:(NSString*)text delay:(double)delay fontSize:(CGFloat)fontSize{
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:[UIApplication sharedApplication].keyWindow animated:YES];
    hud.mode = MBProgressHUDModeText;
    hud.detailsLabelText= text;// 这句支持换行显示
    hud.detailsLabelFont = [UIFont boldSystemFontOfSize:fontSize];
    //    hud.labelText = str;
    
    hud.margin = 14.f;
    hud.removeFromSuperViewOnHide = YES;
    [hud hide:YES afterDelay:delay];
}
- (void)showHint:(NSString *)hint yOffset:(float)yOffset
{
    UIView *view =  [UIApplication sharedApplication].keyWindow;
    //    if([self HUD]==nil){
    //        [self setHUD : [MBProgressHUD showHUDAddedTo:view animated:YES]];
    //        [view addSubview:[self HUD]];
    //    }
    
    if ([hint isKindOfClass:[NSNull class]]) {
        NSLog(@"hint is null......");
        hint = @"";
    }
    
    [self setHUD : [MBProgressHUD showHUDAddedTo:view animated:YES]];
    [view addSubview:[self HUD]];
    [self HUD].userInteractionEnabled = NO;
    // Configure for text only and offset down
    [self HUD].mode = MBProgressHUDModeText;
    [self HUD].labelText = hint;
    [self HUD].margin = 10.f;
    [self HUD].yOffset = 180;
    [self HUD].yOffset += yOffset;
    [self HUD].removeFromSuperViewOnHide = YES;
    
    [[self HUD] hide:YES afterDelay:delay_seconds];
    
}

- (void)hideHud{
    [MBProgressHUD hideHUDForView:self.view animated:YES];
}

#pragma mark -- 20170607 Change by fx
// 20170607 Change
- (MBProgressHUD*)showHUDIndeterminate
{
    return [self showHUDIndeterminateWithText:@"正在加载..."];
}
- (MBProgressHUD*)showHUDIndeterminateWithText:(NSString *)text {
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
    hud.mode = MBProgressHUDModeIndeterminate;
    hud.labelText = text;
    [hud show:YES];
    return hud;
}
- (void)hideHUDIndeterminate {
    [MBProgressHUD hideAllHUDsForView:self.view animated:YES];
}
- (MBProgressHUD*)showHUDWithDetailText:(NSString *)text {
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
    hud.mode = MBProgressHUDModeText;
    hud.detailsLabelText = text;
    [hud hide:YES afterDelay:delay_seconds];
    return hud;
}
- (MBProgressHUD*)showHUDWithText:(NSString *)text detailText:(NSString *)detailText {
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
    hud.mode = MBProgressHUDModeText;
    hud.labelText = text;
    hud.labelFont = [UIFont systemFontOfSize:14.f];
    hud.detailsLabelText = detailText;
    [hud hide:YES afterDelay:delay_seconds];
    return hud;
}
- (MBProgressHUD*)showHUDWithText:(NSString *)text detailText:(NSString *)detailText afterDelay:(float)afterDelay{
    MBProgressHUD *hud = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
    hud.mode = MBProgressHUDModeText;
    hud.labelText = text;
    hud.detailsLabelText = detailText;
    [hud hide:YES afterDelay:afterDelay];
    return hud;
}

// Custom -iamge with title
- (MBProgressHUD*)showHUDWithCustomImage:(NSString *)imageName WithText:(NSString *)text isHaveNavBar:(BOOL )isBar{
    UIImage *image = [[UIImage imageNamed:imageName] imageWithRenderingMode:UIImageRenderingModeAutomatic];
    UIImageView *imageView = [[UIImageView alloc] initWithImage:image];
    MBProgressHUD *hud =[self showHUDWithText:text detailText:nil];
    hud.mode = MBProgressHUDModeCustomView;
    hud.customView = imageView;
    if (isBar) {
        hud.yOffset = -64/2.0; //含导航条 让hud 居中
    }
    //    hud.userInteractionEnabled = NO;
    return hud;
}

@end
